/*
 * @Description: 公共依赖注入
 * @Author: lzc
 * @Date: 2020/12/12 10:20:30
 */
const fs = require('fs');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const { publicUrlOrPath,  resolveApp } = require('./paths');

class HtmlWebpackCommonDepsInject {
  constructor(externals, isEnvProduction) {
    this.externals = Object.keys(externals);
    this.isEnvProduction = isEnvProduction
  }

  apply(compiler) {
    compiler.hooks.compilation.tap('HtmlWebpackThemeInject', compilation => {
      HtmlWebpackPlugin.getHooks(compilation).alterAssetTagGroups.tapAsync('HtmlWebpackThemeInject', (data, callback) => {
        this.externals.forEach(name => {
          const realName = (name || '').replace('/', '-');
          const developmentFileName = `${publicUrlOrPath}/deps/${realName}.js`;
          const productionFileName = `${publicUrlOrPath}/deps/${realName}.min.js`;

          const hasDevelopmentFile = fs.existsSync(resolveApp(`public/deps/${realName}.js`));
          const hasProductionFile = fs.existsSync(resolveApp(`public/deps/${realName}.min.js`));

          if (!hasDevelopmentFile && !hasProductionFile) {
            throw new Error(`common file ${name}  does not in public/deps`);
            process.exit(1);
          }

          const src = this.isEnvProduction
            ? hasProductionFile ? productionFileName : developmentFileName
            : hasDevelopmentFile ? developmentFileName : productionFileName

          data.headTags.push({
            tagName: 'script',
            closeTag: true,
            attributes: {
              src,
              type: 'text/javascript'
            }
          })
        })
        callback(null);
      });
    });
  }
}

module.exports = HtmlWebpackCommonDepsInject;
