/*
 * @Description: xxx
 * @Author: lzc
 * @Date: 2020/7/7 16:17:47
 */
const HtmlWebpackPlugin = require('html-webpack-plugin');

class HtmlWebpackThemeInject {
  constructor(themes = []) {
    this.themes = themes;
    this.excludeinjectTheme = {};
  }

  apply(compiler) {
    compiler.hooks.compilation.tap('HtmlWebpackThemeInject', compilation => {
      // HtmlWebpackPlugin生成资源
      HtmlWebpackPlugin.getHooks(compilation).beforeAssetTagGeneration.tapAsync('HtmlWebpackThemeInject', (data, callback) => {
        if (Array.isArray(data.assets.css)) {
          this.excludeinjectTheme = data.assets.css.filter(name => this.themes.find(v => name.includes(v + '.')))
        }
        callback(null, {...data, assets: {
            ...data.assets,
            css: data.assets.css.filter(v => !this.excludeinjectTheme.includes(v)),
            js: data.assets.js.filter(v => !(this.themes.find(theme => v.includes(theme + '.'))))
          }});
      });

      /**
       * 注入window
       */
      HtmlWebpackPlugin.getHooks(compilation).alterAssetTagGroups.tapAsync('HtmlWebpackThemeInject', (data, callback) => {
        const themeUrl = {};
        this.themes.forEach(name => {
          themeUrl[name] = this.excludeinjectTheme.find(v => v.includes(name + '.'));
        });
        data.headTags.push(
          {
            tagName: 'link',
            closeTag: true,
            attributes: {
              rel: 'stylesheet',
              id: 'theme-css',
              class: 'app-theme-css-link',
              href: themeUrl['default']
            }
          },
          {
            tagName: 'script',
            closeTag: true,
            innerHTML: `
              var APP_THEME_URL = ${JSON.stringify(themeUrl)}
            `,
            attributes: {
              type: 'text/javascript'
            }
          });
        callback(null)
      })
    });
  }
}

module.exports = HtmlWebpackThemeInject;
