/*
 * @Description: xxx
 * @Author: lzc
 * @Date: 2020/12/11 15:09:52
 */
const address = require('address');
const fs = require('fs');
const path = require('path');
const chalk = require('chalk');
const url = require('url');

function onProxyError(proxy) {
  return (err, req, res) => {
    const host = req.headers && req.headers.host;
    console.log(
      chalk.red('Proxy error:') +
      ' Could not proxy request ' +
      chalk.cyan(req.url) +
      ' from ' +
      chalk.cyan(host) +
      ' to ' +
      chalk.cyan(proxy) +
      '.'
    );
    console.log(
      'See https://nodejs.org/api/errors.html#errors_common_system_errors for more information (' +
      chalk.cyan(err.code) +
      ').'
    );
    console.log();

    if (res.writeHead && !res.headersSent) {
      res.writeHead(500);
    }
    res.end(
      'Proxy error: Could not proxy request ' +
      req.url +
      ' from ' +
      host +
      ' to ' +
      proxy +
      ' (' +
      err.code +
      ').'
    );
  };
}


function resolveLoopback(proxy) {
  const o = url.parse(proxy);
  o.host = undefined;
  if (o.hostname !== 'localhost') {
    return proxy;
  }
  try {
    if (!address.ip()) {
      o.hostname = '127.0.0.1';
    }
  } catch (_ignored) {
    o.hostname = '127.0.0.1';
  }
  return url.format(o);
}


function prepareProxy(proxy, appPublicFolder, servedPathname) {
  if (!proxy) {
    return undefined;
  }

  if (Object.prototype.toString.call(proxy) !== '[object Object]') {
    return undefined;
  }

  if (Object.keys(proxy).length === 0) {
    return undefined;
  }

  function mayProxy(pathname) {
    const maybePublicPath = path.resolve(
      appPublicFolder,
      pathname.replace(new RegExp('^' + servedPathname), '')
    );
    const isPublicFileRequest = fs.existsSync(maybePublicPath);
    const isWdsEndpointRequest =
      isDefaultSockHost && pathname.startsWith(sockPath);
    return !(isPublicFileRequest || isWdsEndpointRequest);
  }

  const sockPath = process.env.WDS_SOCKET_PATH || '/sockjs-node';
  const isDefaultSockHost = !process.env.WDS_SOCKET_HOST;

  const result = {};

  Object.entries(proxy).forEach(([match, opts]) => {
    let target;
    if (process.platform === 'win32') {
      target = resolveLoopback(opts.target);
    } else {
      target = opts.target;
    }

    result[match] =  {
      target,
      pathRewrite: opts.pathRewrite,
      logLevel: 'silent',
      secure: false,
      changeOrigin: true,
      ws: true,
      xfwd: true,
      context: (pathname, req) => {
        return (
          req.method !== 'GET' ||
          (mayProxy(pathname) &&
            req.headers.accept &&
            req.headers.accept.indexOf('text/html') === -1)
        );
      },
      onProxyReq: (proxyReq, res) => {
        if (proxyReq.getHeader('origin')) {
          proxyReq.setHeader('origin', target);
        }
      },
      onProxyRes: function onProxyReq(proxyRes, req) {
        proxyRes.headers['x-proxy-by'] = 'reactx-proxy';
        proxyRes.headers['x-proxy-match'] = match;
        proxyRes.headers['x-proxy-target'] = target;

        let distTarget = target;
        if (target && target.endsWith('/')) {
          distTarget = target.replace(/\/$/, '');
        }
        proxyRes.headers['x-proxy-target-path'] = distTarget + req.url;
      },
      onError: onProxyError(target),
    }
  });

  return result;
}

module.exports = prepareProxy;
